#! /usr/bin/env perl
#
# tag: ewok install apache config files installation script
#

use Getopt::Std;
use File::Basename;
use File::Path;
use File::Copy;
use File::Compare;
use strict;

my $build_dir = dirname($0);
my $conf_dir = "$build_dir/../conf";
my $cachefile = "$build_dir/config.cache";
my @conf = qw(ewok.conf ewok-startup.pl);
for (@conf) {
    die "configuration file '$conf_dir/$_' not found" if ! -f "$conf_dir/$_";
}

# Parse options
my %opts;
getopts('n', \%opts);
my $noop = $opts{n};

# Load the cache
die "cannot find cache file $cachefile - have you run configure.pl?" 
    if ! -f $cachefile;
my $cache;
open CACHE, "<", $cachefile
    or die "cannot open '$cachefile' for read";
{
    local $/;
    eval <CACHE>;
}
close CACHE;

# Make apache_conf dir if doesn't exist
if (! -d $cache->{apache_conf}) {
    print "mkdir $cache->{apache_conf}\n";
    if (! $noop) {
        my @new = mkpath($cache->{apache_conf});
        die "creating configuration directory $cache->{apache_conf} failed: $!"
            unless $new[$#new] eq $cache->{apache_conf};
        chown $cache->{httpd_uid}, -1, $cache->{apache_conf}
            if $> == 0 && $cache->{httpd_uid};
    }
}

# Copy files
for (@conf) {
    my $filename = $_;
    if (-f "$cache->{apache_conf}/$_") {
        if (compare("conf/$_", "$cache->{apache_conf}/$_") == 0) {
            print "-> $cache->{apache_conf}/$_ identical (skipping)\n";
            next;
        }
        else {
            print "-> $cache->{apache_conf}/$_ already exists - keeping existing version\n";
            $filename .= '.new';
        }
    }
    print "cp conf/$_ $cache->{apache_conf}/$filename\n";
    if (! $noop) {
        copy("$conf_dir/$_", "$cache->{apache_conf}/$filename");
        chown $cache->{httpd_uid}, -1, "$cache->{apache_conf}/$filename"
            if $> == 0 && $cache->{httpd_uid};
        # Create symlink in ewok_home/conf
        if (! -e "$cache->{ewok_home}/conf") {
            mkpath "$cache->{ewok_home}/conf" 
                or die "mkdir '$cache->{ewok_home}/conf' failed: $!";
            chown $cache->{httpd_uid}, -1, "$cache->{ewok_home}/conf"
                if $> == 0 && $cache->{httpd_uid};
        }
        my $link = "$cache->{ewok_home}/conf/" . basename($_);
        if (! -e $link) {
            symlink "$cache->{apache_conf}/$filename", $link
                or die "symlink $cache->{apache_conf}/$filename, $link failed: $!";
            chown $cache->{httpd_uid}, -1, $link
                if $> == 0 && $cache->{httpd_uid};
        }
    }
}

# Modify httpd.conf, if it exists
my $httpd = "$cache->{apache_conf}/httpd.conf";
if (-f $httpd && -w $httpd) {
    open HTTPD, "<", $httpd
        or die "cannot open '$httpd' for read: $!";
    my $hconf;
    {
        local $/;
        $hconf = <HTTPD>;
    }
    close HTTPD;
    # Add the include for "ewok.conf" unless it's already present
    if ($hconf && $hconf !~ m/^(#\s*)?Include.*\bewok.conf"/m) {
        print "-> adding ewok.conf include directive to $httpd\n";
        if (! $noop) {
            open HTTPD, ">>", $httpd
                or die "cannot open '$httpd' for append $!";
            print HTTPD <<EOD;

# Include ewok configuration directives
Include "ewok.conf"
EOD
            close HTTPD;
        }
    }
    elsif ($1) {
        print "-> $httpd contains ewok.conf include directive,\n   but it's commented out - you'll need to uncomment it to enable ewok\n";
    }
    else {
        print "-> $httpd already contains ewok.conf include directive\n";
    }
}
